-- Codename Dream9 by Basil Voronkov
-- Module 'achievement'
---------------------------------------------
-- This module implements achievements.
require "prefs"
require "lib/const"
require "lib/core"
require "lib/drawing"
require "lib/popup"
require "lib/table"
require "lib/engine"
require "lib/mainmenu"

achievement = {
  arr = {},
  keys = {},
  achivcount = 0,
  x = 10,
  width = 250,
  height = 40,
  mask = "#454545",
  maxop = 240,
  gallerytitleheight = 40,
  gallerytitlefontsize = 22,
  gallerytitlefont = "sans-b.ttf",
  gallerytitlecolor = "red",
  gallerytitlefontcolor = "white"
}

function achievement.init()
  _sysachievement = nil;
  sysachivTotal = 0;
  achievement_gallery = achievement.gallery();
  
  achievement.arr = {};
  achievement.keys = {};
  achievement.register("gamestart", "gamestart", "Пробуждение", "История начинается.");
  achievement.register("help", "help", "Сострадание",      "Катя помогает незнакомцу в метро.");
  achievement.register("night", "night", "Ночной патруль", "Прогулка по улице в комендантский час.");
  achievement.register("tea", "tea", "Чайная церемония", "Катя готовит отцу лечебный чай.");
end

function achievement.register(key, pic, nam, dsc)
  sysachivTotal = sysachivTotal + 1;
  achievement.arr[key] = {pic=pic,nam=nam,dsc=dsc,num=sysachivTotal}
  table.insert(achievement.keys, key);
end

function achievement.unlock(key)
  local k = "achiv_"..key;
  achivcount = nil;
  if not prefs[k] then
    prefs[k] = true;
    _sysachievement = key;
  end
end

function achievement.unlocked()
  if achivcount == nil then
    achivcount = 0;
    for k,_ in pairs(prefs) do
      if k:startsWith("achiv_") and prefs[k] then
        achivcount = achivcount + 1;
      end
    end
  end
  return achivcount;
end

function achievement.gallery()
  local r = engine.gameroom {
     img        = mainmenu.image
    ,gen_       = true
    ,hideinv    = true
    ,fps        = 20
    ,nomask     = true
    ,ondrawbg   = function(s, p)
                    if s.total == nil then
                      s:fetch();
                    end
                    --drawing.draw_mask(p, 0, 0, const.picwidth, const.picheight, "black", 150);
                    drawing.draw_mask(p, 0, 10, const.picwidth, achievement.gallerytitleheight, achievement.gallerytitlecolor, 150);
                    drawing.draw_text(p, "Достижения ("..tostring(s.total).."/"..tostring(table.len(achievement.arr))..")", 10, 10 + (achievement.gallerytitleheight - 
                      drawing.text_height(achievement.gallerytitlefont, achievement.gallerytitlefontsize))/2,
                      achievement.gallerytitlefontcolor, achievement.gallerytitlefont, achievement.gallerytitlefontsize);
                    drawing.draw_pic(p, const.path.ui.."close.png", const.picwidth - 20, 20);
                    --if s.total == 0 then
                    --  drawing.draw_text(p, "Разблокированных достижений нет.", 10, s.miny);
                    --end
                    return p;
                  end
    ,scrolly    = achievement.gallerytitleheight + 10 + 30
    ,miny       = achievement.gallerytitleheight + 10 + 30
    ,fetch      = function(s)
                    s.total = achievement.unlocked();
                    s.maxachiv = table.len(achievement.arr);
                  end
    ,ondraw     = function(s, p)
                    if s.total == nil then
                      s:fetch();
                    end
                    local y = s.scrolly;
                    local x = 20;
                    drawing.clean(p, 0, 0, const.picwidth, const.picheight);
                    local drawn = 0;
                    s.first = nil;
                    s.last = 0;
                    
                    for i,k in ipairs(achievement.keys) do
                      local a = achievement.arr[k];
                      local w,h = achievement.width+30, achievement.height;
                      local ao = 255;
                      if not prefs["achiv_"..k] then
                        ao = 100;
                      end
                      if y >= s.miny and drawn < 8*3 then
                        local b = sprite.box(w, h, achievement.mask, 255);
                        drawing.draw_pic(b, const.path.achiv..a.pic..".png", 4, (h-32)/2);
                        local nm = a.nam;
                        if ao ~= 100 then
                          nm = a.nam.." ("..a.num..")";
                        end
                        drawing.draw_text(b, nm, 42, 4, const.textcolor, const.fontb, const.fontsize);
                        if ao ~= 100 then
                          drawing.draw_text(b, a.dsc, 42, 22, const.textcolor, const.font, const.fontsize);
                        end
                        sprite.copy(b, 0, 0, w, h, p, x, y, ao);
                        sprite.free(b);
                        drawn = drawn + 1;
                        if s.first == nil then
                          s.first = i;
                        end
                        s.last = i;
                      end
                      x = x + w + 20;
                      if x + w > const.picwidth then
                        y = y + h + 20;
                        x = 20;
                      end
                    end
                    if s.first == nil then
                      s.first = 1;
                    end
                    local pic2 = "up";
                    local pic3 = "down";
                    if s.sel == 2 then
                      pic2 = "up_sel";
                    elseif s.sel == 3 then
                      pic3 = "down_sel";
                    end
                    if s.first ~= 1 then
                      drawing.copy_pic(p, const.path.ui..pic2..".png", const.picwidth - 40, achievement.gallerytitleheight + 10 + 30);
                    end
                    if s.last ~= s.maxachiv then
                      drawing.copy_pic(p, const.path.ui..pic3..".png", const.picwidth - 40, const.picheight - 30);
                    end
                  end
    ,onclick    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel == 1 then
                      walkback();
                    elseif sel == 2 and s.first ~= 1 then
                      s.scrolly = s.scrolly + (achievement.height + 20)*8;
                      s:redraw();
                    elseif sel == 3 and s.last ~= s.total then
                      s.scrolly = s.scrolly - (achievement.height + 20)*8;
                      s:redraw();
                    end
                  end
    ,selected   = function(s, x, y)
                    y = y - const.blackbar;
                    if x < const.picwidth - 40 then
                      return nil;
                    end
                    if y > 10 and y < achievement.gallerytitleheight+10 then
                      return 1;
                    elseif y >= achievement.gallerytitleheight + 40 and y < achievement.gallerytitleheight + 60 then
                      return 2;
                    elseif y >= const.picheight - 50 then
                      return 3;
                    end
                  end
    ,ontimer    = function(s, x, y)
                    local pic1 = "close";
                    local pic2 = "up";
                    local pic3 = "down";
                    
                    local sel = s:selected(x, y);
                    if sel == 1 then
                      pic1 = "close_sel";
                    elseif sel == 2 then
                      pic2 = "up_sel";
                    elseif sel == 3 then
                      pic3 = "down_sel";
                    end
                    if sel ~= s.sel then
                      drawing.copy_pic(s.pic, const.path.ui..pic1..".png", const.picwidth - 20, 20);
                      if s.first ~= 1 then
                        drawing.copy_pic(s.pic, const.path.ui..pic2..".png", const.picwidth - 40, achievement.gallerytitleheight + 10 + 30);
                      end
                      if s.last ~= s.maxachiv then
                        drawing.copy_pic(s.pic, const.path.ui..pic3..".png", const.picwidth - 40, const.picheight - 30);
                      end
                      s.sel = sel;
                    end
                  end
  }
  r.invalidate  = function(s)
                    s._height = 0;
                    theme.set("scr.gfx.h", const.picwidth);
                  end
  return r;
end

function achievement.room(o)
  local ontimer = o.ontimer;
  local ondraw = o.ondraw;
  local function draw_achiv(s, ao)
    local y = 12;
    if _syspopup ~= nil then
      y = 40
    end
    local a = achievement.arr[_sysachievement];
    drawing.clean(s.pic, achievement.x, y, achievement.width, achievement.height);
    local b = sprite.box(achievement.width, achievement.height, achievement.mask, 255);
    drawing.draw_pic(b, const.path.achiv..a.pic..".png", 4, (achievement.height-32)/2);
    drawing.draw_text(b, "Открыто достижение (" .. tostring(achievement.unlocked()) .. "/" .. tostring(sysachivTotal) .. ")", 
      42, 4, const.textcolor, const.fontb, const.fontsize);
    drawing.draw_text(b, a.nam, 42, 22, const.textcolor, const.font, const.fontsize);
    sprite.copy(b, 0, 0, achievement.width, achievement.height, s.pic, achievement.x, y, ao);
    sprite.free(b);
  end
  o.ondraw      = function(s, pic)
                    if ondraw ~= nil then
                      ondraw(s, pic);
                    end
                    --if _sysachievement ~= nil and s.ao_ == 255 then
                    --  draw_achiv(s, 255);
                   -- end
                  end
  o.ontimer     = function(s, x, y)
                    if ontimer ~= nil then
                      ontimer(s, x, y)
                    end
                    if _sysachievement ~= nil then
                      if s.ao_ ~= achievement.maxop then
                        if s.ao_ == nil then
                          s.ao_ = 0
                          play_sound("achiv");
                        end
                        draw_achiv(s, s.ao_);
                        s.ao_ = s.ao_ + 20;
                      else
                        _sysachievement = nil;
                        s.ao_ = nil;
                      end
                    end
                  end
  return o;
end