-- Codename Dream9 by Basil Voronkov
-- Module 'anitext'
---------------------------------------------
-- Implements text animations.
require "lib/core"
require "lib/table"
require "lib/drawing"
require "lib/engine"
require "theme"

anitext = {}
anitext.x = 260
anitext.y = 180
anitext.font = "UbuntuMono-R.ttf"
anitext.fontsize = 18
anitext.fontcolor = "#00FF00";
anitext.linespace = 2;
anitext.menux = anitext.x+5;
anitext.menuspace = 25;
anitext.menuitemspace = 15;
anitext.menuselspacex = 2;
anitext.menuselspacey = 2;
anitext.bg = "black";

function anitext.room(o)
  return engine.gameroom {
     fps        = 40
    ,hideinv    = true
    ,_count     = 0
    ,_textx     = anitext.x
    ,_texty     = anitext.y
    ,img        = "gfx/black.jpg"
    ,dsc_       = o.dsc
    ,len_       = string.unilen(o.dsc)
    ,items      = o.items
    ,_itemcount = 0
    ,itemlen_   = table.len(o.items)
    ,enter      = function(s)
                    if not s._done then
                      play_sound("terminal");
                    end
                  end
    ,ontimer    = function(s, x, y)
                    if not s._done then
                      s:animate();
                    else
                      local fn,fs,col = anitext.font,anitext.fontsize,anitext.fontcolor;
                      local st = drawing.text_height(fn, fs) + anitext.menuitemspace;
                      local my = s:selected(x, y);
                      if my ~= s.sel and s.oldi ~= nil then
                        drawing.clean(s.pic, anitext.x, s.oldy, s.oldw, s.oldh);
                        drawing.copy_text(s.pic, tostring(s.oldi)..". "..s.items[s.oldi][1], 
                          anitext.menux, s.oldy+anitext.menuselspacey, col, fn, fs);
                      end
                      if my > 0 and s.items[my] ~= nil then
                        local t = tostring(my)..". "..s.items[my][1];
                        s.oldy = s._first+st*(my-1)-anitext.menuselspacey;
                        s.oldw = s._max - anitext.x;
                        s.oldh = st-anitext.menuitemspace+anitext.menuselspacey*2;
                        s.oldi = my;
                        drawing.copy_mask_generic(s.pic, anitext.x, s.oldy, s.oldw, s.oldh, col, 255);
                        drawing.copy_text(s.pic, t, anitext.menux, s.oldy+anitext.menuselspacey, anitext.bg, fn, fs);
                      end
                      s.sel = my;
                    end
                  end
    ,onclick    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if s._done and sel ~= nil and s.items[sel] ~= nil then
                      local act = s.items[sel][2];
                      if is_function(act) then
                        act();
                      else
                        exec(act);
                      end
                    end
                  end
    ,selected   = function(s, x, y)
                    if s._done then
                      local st = drawing.text_height(anitext.font, anitext.fontsize) + anitext.menuitemspace;
                      return math.ceil((y - s._first)/ st);
                    end
                  end
    ,ondraw     = function(s, p)
                    if s._done then
                      local fn,fs,col = anitext.font,anitext.fontsize,anitext.fontcolor;
                      local arr = s.dsc_:split("^");
                      local y = anitext.y;
                      for v in arr do
                        drawing.copy_text(p, v, anitext.x, y, col, fn, fs);
                        y = y + drawing.text_height(anitext.font, anitext.fontsize) + anitext.linespace;
                      end
                      y = s._first;
                      for i,v in ipairs(s.items) do
                        local txt = tostring(i)..". "..v[1];
                        drawing.copy_text(p, txt, anitext.menux, y, col, fn, fs);
                        y = y + drawing.text_height(fn, fs) + anitext.menuitemspace;
                      end
                    end
                  end
    ,animate    = function(s)
                    local fn,fs,col = anitext.font,anitext.fontsize,anitext.fontcolor;
                    if s._count < s.len_ then
                      s._count = s._count + 1;
                      local ch = s.dsc_:charat(s._count);
                      if ch ~= "^" then
                        local tw = drawing.text_width(ch, fn, fs);
                        drawing.copy_text(s.pic, ch, s._textx, s._texty, col, fn, fs);
                        s._textx = s._textx + tw;
                      else
                        if s._max == nil or s._textx > s._max then
                          s._max = s._textx;
                        end
                        s._textx = anitext.x;
                        s._texty = s._texty + drawing.text_height(fn, fs) + anitext.linespace;
                      end
                    elseif s._itemcount < s.itemlen_ then
                      if s._max == nil then
                        s._max = s._textx;
                      end
                      s._texty = s._texty + drawing.text_height(fn, fs) + anitext.menuitemspace;
                      if s._itemcount == 0 then
                        play_sound("click");
                        s._texty = s._texty + anitext.menuspace;
                        s._first = s._texty;
                      end
                      s._itemcount = s._itemcount + 1;
                      local txt = tostring(s._itemcount)..". "..s.items[s._itemcount][1];
                      drawing.copy_text(s.pic, txt, anitext.menux, s._texty, col, fn, fs);
                    else
                      s._done = true;
                    end
                  end
  }
end

function anitext.title(o)
  local r = engine.gameroom {
     img        = const.path.gfx.."black.jpg"
    ,title      = o.title
    ,fontsize   = o.fontsize
    ,ondrawbg   = function(s, p)
                    local fn,fs = const.fontb,s.fontsize;
                    local w,h = drawing.text_width(s.title, fn, fs),drawing.text_height(fn, fs);
                    drawing.draw_text(p, s.title, (const.picwidth-w)/2, (const.picheight-h)/2, "white", fn, fs);
                    return p;
                  end
    ,fading     = o.fading
    ,fps        = 1000/o.time
    ,next_      = o.next
    ,entered    = o.entered
    ,enter      = o.enter
    ,hideinv    = true
    ,ontimer    = function(s)
                    if is_function(s.next_) then
                      s:next_();
                    else
                      walk(s.next_);
                    end
                  end
  }
  r.invalidate  = function(s)
                    s._height = 0;
                    s._top = 0;
                  end
  return r;
end