-- Codename Dream9 by Basil Voronkov
-- Module 'engine'
---------------------------------------------
-- Core game engine
require "theme"
require "click"
require "timer"
require "lib/const"
require "lib/core"
require "lib/table"
require "lib/drawing"

engine = {
  seq = 0
}

function engine.xacts(o)
  return txt:parse(function(s) 
    local arr = s:split("|");
    if table.len(arr) > 1 then
      return "{"..s.."}";
    else
      return "{"..deref(o).."|"..s.."}";
    end
  end);
end

function engine.extract_dsc(s)
  local dsc = "";
  local cc = 0;
  for i,v in ipairs(objs(s)) do
    if v.dsc_scn ~= nil and (v.cnd == nil or rcall(v.cnd, v)) then
      local d = rcall(v.dsc_scn, v);
      if d ~= nil and d ~= "" then
        if cc > 0 then
          dsc = dsc.." ";
        end
        cc = cc + 1;
        dsc = dsc..d;
      end
    end
  end
  if dsc ~= "" then
    return dsc;
  end
end

function engine.previous_pic(s)
  local ret = nil;
  local so = s;
  while true do
    if so.__from__ == nil then
      return const.path.gfx.."black.jpg";
    end
    ret = ref(so.__from__).img;
    so = ref(so.__from__);
    if not so.gen_ then
      return ret;
    end
  end
end

function engine.main(v)
  return room {
     _seen      = false
    ,nam        = "main"
    ,start      = v.start
    ,finish     = v.finish
    ,enter      = function(s)
                    if not s._seen then
                      s._seen = true;
                      rcall(s.start, s);
                    else
                      rcall(s.finish, s);
                      return false;
                    end
                  end
  }
end

function engine.state(o)
  o.nam = "state";
  return obj(o);
end

function engine.hub(o)
  local enter = o.enter;
  return room {
     nam        = "hub"
    ,enter      = function(s)
                    enter(s);
                    s._seen = true;
                  end
  }
end

function engine.gameroom(o)
  local enter = o.enter;
  local exit = o.exit;
  local dsc = o.dsc;
  engine.seq = engine.seq + 1;
  local ret = room {
     nam        = function(s)
                    if s.redraw ~= nil then
                      s:redraw();
                    end
                    return "";
                  end
    ,nomask     = o.nomask
    ,gamekey    = engine.seq
    ,pic        = drawing.sprite_blank(const.picwidth, const.picheight, true)
    ,obj        = o.obj
    ,invalidate = function(s)
                    local d = s:dsc();
                    local h = 0;
                    if d ~= nil then
                      d = d:normalize();
                    end
                    if d ~= nil and d ~= "" then
                      h = drawing.text_bounds(d, const.textwidth, const.scenefont, const.scenefontsize);
                      local pad = const.blackbar;
                      s._height = h + const.dscframepady*2;
                      s._top = const.winheight - s._height - pad;
                      theme.win.font(const.path.theme..const.scenefont, const.scenefontsize, 1.0)
                    else
                      s._height = 0;
                      s._top = 0;
                    end
                    theme.win.geom(const.dscpad, s._top, const.textwidth, 800);--h);
                    if s._top <= 0 then
                      s._top = const.winheight-const.blackbar;
                    end
                    local gfxh = s._top - const.blackbar;
                    if gfxh < 0 then
                      gfxh = 0;
                    end
                    theme.set("scr.gfx.h", gfxh);
                  end
    ,dsc        = function(s)
                    local xacts = engine.extract_dsc(s);
                    local ret = dsc;
                    if ret ~= nil and (s.showdsc == nil or s:showdsc()) then
                      ret = rcall(ret, s);
                      if xacts ~= nil and ret ~= nil and ret ~= "" then
                        ret = ret.."^"..xacts;
                      elseif xacts ~= nil then
                        ret = xacts;
                      end
                    else
                      ret = xacts;
                    end
                    return ret;
                  end
    ,drawbg     = function(s)
                    local ht = false;
                    if s.invalidate ~= nil then
                      ht = true;
                      s:invalidate();
                    end
                    local dsc = nil;
                    if s.dsc ~= nil then
                      dsc = s:dsc();
                    else
                      ht = false;
                    end
                    local pf = rcall(s.img, s);
                    if pf ~= nil then
                      local pic = sprite.load(pf);
                      if ht and dsc ~= nil and dsc ~= "" and not s.nomask then
                        drawing.draw_mask(pic, const.dscframepadx, 
                          const.picheight - s._height - const.dscframepady,
                          const.textframewidth, s._height);
                      end
                      if s.ondrawbg ~= nil then
                        pic = s:ondrawbg(pic);
                      end
                      theme.gfx.bg(pic);
                    end
                  end
    ,draw       = function(s)
                    s:clean();
                    if s.ondraw ~= nil then
                      s:ondraw(s.pic);
                    end
                  end
    ,clean      = function(s)
                    drawing.clean(s.pic, 0, 0, const.picwidth, const.picheight);
                  end
    ,redraw     = function(s)
                    s:drawbg();
                    s:draw();
                    if s.onredraw ~= nil then
                      s:onredraw();
                    end
                  end
    ,enter      = function(s)
                    if s.fps ~= nil and s.fps > 0 then
                      timer:set(1000/s.fps);
                    end
                    if enter ~= nil then
                      enter(s);
                    end
                  end
    ,entered    = o.entered
    ,exit       = function(s)
                    if s.fps ~= nil and s.fps > 0 then
                      timer:stop();
                    end
                    if exit ~= nil then
                      exit(s);
                    end
                  end
    ,timer      = function(s)
                    if s.ontimer ~= nil then
                      local x,y = mouse_pos();
                      s:ontimer(x, y);
                    end
                  end
    ,click      = function(s, button)
                    if s.onclick ~= nil and button == 1 then
                      local x,y = mouse_pos();
                      s:onclick(x, y);
                    end
                    if s.onclickr ~= nil and button == 3 then
                      local x,y = mouse_pos();
                      s:onclickr(x, y);
                    end
                  end
  }
  copy_attr(o, ret);
  return ret;
end

function engine.item(o)
  local ret = obj {
     nam        = o.nam
    ,obj        = o.obj
    ,dsc_       = o.dsc
    ,dsc_scn    = function(s)
                    local txt = rcall(s.dsc_, s);
                    if txt ~= nil then
                      return txt:parse(function(st) 
                        local arr = st:split("|");
                        local c = 0;
                        for i,v in arr do
                          c = c + 1;
                        end
                        if c > 1 then
                          return "{"..st.."}";
                        else
                          return "{"..deref(s).."|"..st.."}";
                        end
                      end);
                    end
                    return txt;
                  end
  }
  copy_attr(o, ret);
  ret.dsc = nil;
  return ret;
end