-- Codename Dream9 by Basil Voronkov
-- Module 'episodes'
---------------------------------------------
-- Implements check points engine and an ability to
-- load an episode from a menu.
require "lib/core"
require "lib/const"
require "lib/mainmenu"

episodes = {
  arr = {},
  
  x = 10,
  width = 370,
  height = 68,
  mask = "black",
  maxop = 240,
  itemx = 50,
  itemy = 40 + 10 + 50,
  gallerytitleheight = 40,
  gallerytitlefontsize = 22,
  gallerytitlefont = "sans-b.ttf",
  gallerytitlecolor = "red",
  gallerytitlefontcolor = "white"
}

function episodes.init()
  episodes.arr = {};
  episodes.register("0-intro", "petr", "Пролог", "Выберите один из цветов.", "0-prolog/intro.lua");
  episodes.register("0-escape", "kate", "Пролог", "Катя спасается из горящего здания.", "0-prolog/escape.lua");
  episodes.register("1-wake", "kate", "Мертвый час", "Катя просыпается среди ночи.", "1-deadhour/wake.lua");
  episodes.register("1-med", "kate", "Мертвый час", "Катя в центре здоровья.", "1-deadhour/med.lua");
  episodes.register("1-institute", "kate", "Мертвый час", "Катя в институте онкологии.", "1-deadhour/institute.lua");
  episodes.register("1-subway", "kate", "Мертвый час", "Катя возвращается домой на метро.", "1-deadhour/subway.lua");
  episodes.register("1-night", "kate", "Мертвый час", "В городе начался комендантский час.", "1-deadhour/night.lua");
  episodes.register("1-clinic", "kate", "Мертвый час", "Катя привела отца на обследование.", "1-deadhour/clinic.lua");
  episodes.register("1-tea", "kate", "Мертвый час", "Катя дома ухаживает за отцом.", "1-deadhour/tea.lua");
    
  sys_episodes = episodes.gallery();
end

function episodes.register(key, img, nam, dsc, gamefile)
  table.insert(episodes.arr, {key=key,img=img,nam=nam,dsc=dsc,gamefile=gamefile});
end

function episodes.unlock(key)
  prefs["episode_"..key] = true;
  local s = nil;
  for _,v in ipairs(episodes.arr) do
    if v.key == key then
      prefs["last_episode"] = v.gamefile;
    end
  end
end

function episodes.gallery()
  local r = engine.gameroom {
     img        = mainmenu.image
    ,gen_       = true
    ,hideinv    = true
    ,fps        = 20
    ,extrac     = 0
    ,nomask     = true
    ,ondrawbg   = function(s, p)
                    if s.total == nil then
                      s:fetch();
                    end
                    --drawing.draw_mask(p, 0, 0, const.picwidth, const.picheight, "black", 150);
                    drawing.draw_mask(p, 0, 10, const.picwidth, episodes.gallerytitleheight, episodes.gallerytitlecolor, 150);
                    drawing.draw_text(p, "Выбор эпизода", 10, 10 + (episodes.gallerytitleheight - 
                      drawing.text_height(episodes.gallerytitlefont, episodes.gallerytitlefontsize))/2,
                      episodes.gallerytitlefontcolor, episodes.gallerytitlefont, episodes.gallerytitlefontsize);
                    drawing.draw_pic(p, const.path.ui.."close.png", const.picwidth - 20, 20);
                    if s.total == 0 then
                      drawing.draw_text(p, "Открытых эпизодов нет.", 10, s.miny);
                    end
                    return p;
                  end
    ,scrolly    = episodes.itemy
    ,miny       = episodes.itemy
    ,fetch      = function(s)
                    s.arr = {};
                    s.total = 0;
                    for i,v in pairs(episodes.arr) do
                      if prefs["episode_"..v.key] then
                        table.insert(s.arr, v);
                        s.total = s.total + 1;
                      end
                    end
                  end
    ,drawitem   = function(s, p, a, x, y, alpha)
                    local w,h = episodes.width+30, episodes.height;
                    drawing.clean(p, x, y, w, h);
                    local b = sprite.box(w, h, episodes.mask, 100);
                    sprite.copy(b, 0, 0, w, h, p, x, y, 255);
                    sprite.free(b);
                    drawing.copy_pic(p, const.path.gfx.."episodes/"..a.img..".png", x+4, y+((h-64)/2), false, alpha);
                    drawing.copy_text(p, a.nam, x+74, y+4, const.textcolor, const.fontb, 16, alpha);
                    drawing.copy_text(p, a.dsc, x+74, y+30, const.textcolor, const.font, const.fontsize, alpha);
                  end
    ,ondraw     = function(s, p)
                    if s.total == nil then
                      s:fetch();
                    end
                    local y = s.scrolly;
                    local x = episodes.itemx;
                    local w,h = episodes.width+30, episodes.height;
                    drawing.clean(p, 0, 0, const.picwidth, const.picheight);
                    local drawn = 0;
                    s.first = nil;
                    s.last = 0;
                    for i,a in ipairs(s.arr) do
                      local k = a.key;
                      if y >= s.miny and drawn < 5*2 then
                        s:drawitem(p, a, x, y, 100);
                        drawn = drawn + 1;
                        if s.first == nil then
                          s.first = i;
                        end
                        s.last = i;
                      end
                      x = x + w + 20;
                      if x + w > const.picwidth then
                        y = y + h + 20;
                        x = episodes.itemx;
                      end
                    end
                    if s.first == nil then
                      s.first = 1;
                    end
                    local pic2 = "up";
                    local pic3 = "down";
                    if s.sel == 2 then
                      pic2 = "up_sel";
                    elseif s.sel == 3 then
                      pic3 = "down_sel";
                    end
                    if s.first ~= 1 then
                      drawing.copy_pic(p, const.path.ui..pic2..".png", const.picwidth - 60, episodes.itemy);
                    end
                    if s.last ~= s.total then
                      drawing.copy_pic(p, const.path.ui..pic3..".png", const.picwidth - 60, const.picheight - 50);
                    end
                  end
    ,onclick    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel == 1 then
                      walkback();
                    elseif sel == 2 and s.first ~= 1 then
                      s.ep_oldsel = nil;
                      s.scrolly = s.scrolly + (episodes.height + 20)*5;
                      s.extrac = s.extrac - 5*2;
                      s:redraw();
                    elseif sel == 3 and s.last ~= s.total then
                      s.ep_oldsel = nil;
                      s.scrolly = s.scrolly - (episodes.height + 20)*5;
                      s.extrac = s.extrac + 5*2;
                      s:redraw();
                    elseif sel ~= nil and sel < 0 then
                      gamefile(const.path.game..s.arr[0-sel].gamefile, true);
                    end
                  end
    ,selected   = function(s, x, y)
                    y = y - const.blackbar;
                    if x < const.picwidth - 90 and x >= episodes.itemx then
                      if y >= episodes.itemy and y <= const.picheight - 50 then
                        y = y - episodes.itemy;
                        local row = math.ceil(y / (episodes.height+20));
                        x = x - episodes.itemx;
                        local col = math.ceil(x / (episodes.width+30+20));                        
                        return 0 - ((row-1)*2 + col);
                      end
                    else
                      if y > 10 and y < episodes.gallerytitleheight+10 then
                        return 1;
                      elseif y >= episodes.itemy and y < episodes.itemy + 25 then
                        return 2;
                      elseif y >= const.picheight - 60 then
                        return 3;
                      end
                    end
                  end
    ,ontimer    = function(s, x, y)
                    local pic1 = "close";
                    local pic2 = "up";
                    local pic3 = "down";
                    
                    local sel = s:selected(x, y);
                    if sel == 1 then
                      pic1 = "close_sel";
                    elseif sel == 2 then
                      pic2 = "up_sel";
                    elseif sel == 3 then
                      pic3 = "down_sel";
                    end
                    if sel ~= s.sel then
                      drawing.copy_pic(s.pic, const.path.ui..pic1..".png", const.picwidth - 20, 20);
                      if s.first ~= 1 then
                        drawing.copy_pic(s.pic, const.path.ui..pic2..".png", const.picwidth - 60, episodes.itemy);
                      end
                      if s.last ~= s.total then
                        drawing.copy_pic(s.pic, const.path.ui..pic3..".png", const.picwidth - 60, const.picheight - 50);
                      end
                      local w,h = episodes.width+30,episodes.height;
                      if s.ep_oldsel ~= nil then
                        drawing.clean(s.pic, s.ep_oldx, s.ep_oldy, w, h);
                        s:drawitem(s.pic, s.arr[s.ep_oldsel], s.ep_oldx, s.ep_oldy, 100);
                        s.ep_oldsel = nil;
                      end
                      if sel~= nil and sel < 0 then
                        local pos = (0-sel)+s.extrac;
                        local even = math.ceil(pos/2) == pos/2;
                        local x,y = 0,0;
                        if not even then
                          x = episodes.itemx;
                          y = ((((0-sel)+1)/2)-1)*(h+20)+episodes.itemy;
                        else
                          x = episodes.itemx + w+20;
                          y = (((0-sel)/2)-1)*(h+20)+episodes.itemy;
                        end 
                        if s.arr[pos] ~= nil then
                          s:drawitem(s.pic, s.arr[pos], x, y, 255);
                          s.ep_oldx = x;
                          s.ep_oldy = y;
                          s.ep_oldsel = pos;
                        end
                      end
                      s.sel = sel;
                    end
                  end
  }
  r.invalidate  = function(s)
                    s._height = 0;
                    theme.set("scr.gfx.h", const.picwidth);
                  end
  return r;
end