-- Codename Dream9 by Basil Voronkov
-- Module 'inventory'
---------------------------------------------
-- This module implements an in game inventory with
-- a dedicated inventory screen.
require "lib/core"
require "lib/drawing"
require "lib/table"
require "lib/engine"
require "lib/const"

inventory = {
  defaults = {
    menuwidth = 200,
    itempad = 10,
    itemsize = 32,
    itemspace = 15,
    font = const.fontb
  }
}

function inventory.count()
  return table.len(_inventory);
end

function inventory.visible()
  return inventory.count() > 0;
end

function inventory.add(o)
  table.insert(_inventory, deref_if(o));
end

function inventory.at(n)
  return ref(_inventory[n]);
end

function inventory.remove(o)
  local nm = deref_if(o);
  for i,v in ipairs(_inventory) do
    if v == nm then
      return table.remove(_inventory, i);
    end
  end
end

function inventory.has(nm)
  if is_table(nm) then
    nm = deref(nm);
  end
  for i,v in ipairs(_inventory) do
    if v == nm then
      return true;
    end
  end
end

function inventory.menuitem()
  return menu {
    nam         = function(s)
                    local hw = here();
                    if not hw.hideinv then
                      local len = inventory.count()
                      if inventory.count() > 0 then
                        if hw.inventory_ then
                          return img(const.path.ui.."inv_opened.bmp")..txtb("Инвентарь ("..tostring(len)..")")..txtnb(" ");
                        else
                          return img(const.path.ui.."inv_closed.bmp")..txtb("Инвентарь ("..tostring(len)..")")..txtnb(" ");
                        end
                      end
                    end
                  end
  ,menu         = function(s)
                    local hw = here();
                    if here().inventory_ then
                      walkback(_inventoryback);
                    else
                      _inventoryback = deref(hw);
                      walk("sys_inventoryroom");
                    end
                  end
  }
end

function inventory.room()
  return engine.gameroom {
     img        = engine.previous_pic
    ,inventory_ = true
    ,gen_       = true
    ,fps        = 20
    ,ontimer    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel ~= s.sel then
                      s.sel = sel;
                      s:ondraw(s.pic);
                    end
                  end
    ,selected   = function(s, x, y)
                    local w,h = inventory.defaults.menuwidth,const.picheight;
                    y = y - 20;
                    local sel = nil;
                    if x > w or y < 0 then
                      sel = nil;
                    else
                      local step = inventory.defaults.itemsize + inventory.defaults.itemspace;
                      sel = math.ceil(y/step);
                    end
                    return sel;
                  end
    ,onclick    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel ~= nil then
                      local o = inventory.at(sel);
                      walk(_inventoryback);
                      o:activate();
                    end
                  end
    ,ondrawbg   = function(s, p)
                    if from().drawtiles ~= nil then
                      p = from():drawtiles(p);
                    end
                    local w,h = inventory.defaults.menuwidth,const.picheight;
                    drawing.draw_mask(p, 0, 0, w, h);
                    return p;
                  end
    ,ondraw     = function(s, pic)
                    local w,h = inventory.defaults.menuwidth,const.picheight;
                    local start = inventory.defaults.itempad;
                    local pad = start;
                    local fn = inventory.defaults.itemfont;
                    local size = inventory.defaults.itemsize;
                    local step = inventory.defaults.itemsize + inventory.defaults.itemspace;
                    
                    if s._lasty ~= nil then
                      drawing.clean(pic, w, s._lasty, const.picwidth-w, inventory.defaults.itemsize);
                    end
                    
                    if s.sel ~= nil then
                      local o = inventory.at(s.sel);
                      if o ~= nil then
                        local y = (s.sel-1)*step + pad;
                        drawing.textframe(pic, rcall(o.inf, o), w + 10, y + ((size-drawing.text_height())/2) - 4, 4, 4);
                        s._lasty = y;
                      end
                    end
                    
                    for i,v in ipairs(_inventory) do
                      local e = ref(v);
                      local col = "white";
                      drawing.clean(pic, 0, start, w, size);
                      if s.sel == i then
                        col = const.activelinkcolor;
                      end
                      drawing.copy_pic(pic, rcall(e.ico, e), 10, start);
                      drawing.copy_text(pic, rcall(e.nam, e), size+20, start+((size-drawing.text_height())/2), col, inventory.defaults.font);
                      start = start + step;
                    end
                  end
  }
end

function inventory.init()
  sys_inventoryroom = inventory.room();
  inventory_object = inventory.menuitem();
  take(inventory_object);
  _inventory = {};
  _inventoryback = nil;
end