-- Codename Dream9 by Basil Voronkov
-- Module 'mainmenu'
---------------------------------------------
-- Implements main menu screens
require "lib/core"
require "lib/const"
require "lib/drawing"
require "lib/engine"

mainmenu = {
  width = 200,
  itemheight = 32,
  itemspace = 10,
  itempad = 10,
  itemfont = "sans-b.ttf",
  itemfontsize = 14,
}

function mainmenu.init()
  _menuback = nil;
  menu_object = mainmenu.menuitem();
  take(menu_object);
  sys_menuroom = mainmenu.titlescreen {
     [1]        = {"Продолжить игру", "Продолжить начатую игру", mainmenu.resume, mainmenu.canresume}
    ,[2]        = {"", "", nil, mainmenu.canresume}
    ,[3]        = {"Новая игра", "Начать новую игру", gamefile_("0-prolog/intro.lua")}
    ,[4]        = {"Выбор эпизода", "Начать игру с одной из пройденных глав", walk_("sys_episodes")}
    ,[5]        = {"Достижения", "Посмотреть список достижений", walk_("achievement_gallery")}
  }
  gameover = mainmenu.gameover();
end

function mainmenu.canresume()
  return _menuback ~= nil or prefs["last_episode"] ~= nil;
end

function mainmenu.resume()
  if _menuback ~= nil then
    local mb = _menuback;
    _menuback = nil;
    if ref(mb).inventory_ then
      walk(_inventoryback);
    else
      walk(mb);
    end
  elseif prefs["last_episode"] ~= nil then
    gamefile(const.path.game..prefs["last_episode"], true);
  end
end

function mainmenu.image()
  if _menuback == nil then
    return const.path.gfx.."titlescreen.jpg";
  end
  return engine.previous_pic;
end

function mainmenu.menuitem()
  return menu {
    nam         = function(s)
                    local hw = here();
                    if not hw.hideinv then
                      return txtnb(" ")..txtb("Меню");
                    end
                  end
  ,menu         = function(s)
                    local hw = here();
                    _menuback = deref(hw);
                    walk("sys_menuroom");
                  end
  }
end

function mainmenu.titlescreen(obj)
  return engine.gameroom {
     img        = mainmenu.image
    ,gen_       = true
    ,hideinv    = true
    ,fps        = 20
    ,ontimer    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel ~= s.sel then
                      s.sel = sel;
                      s:ondraw(s.pic);
                    end
                  end
    ,selected   = function(s, x, y)
                    local w,h = mainmenu.width,const.picheight;
                    y = y - 20;
                    local sel = nil;
                    if x > w or y < 0 then
                      sel = nil;
                    else
                      local step = mainmenu.itemheight + mainmenu.itemspace;
                      sel = math.ceil(y/step);
                    end
                    local e = obj[sel];
                    if e ~= nil and e[1] ~= "" and (e[4] == nil or rcall(e[4],e)) then
                      return sel;
                    end
                  end
    ,onclick    = function(s, x, y)
                    local sel = s:selected(x, y);
                    if sel ~= nil then
                      local o = obj[sel][3];
                      s.sel = nil;
                      o();
                    end
                  end
    ,ondrawbg   = function(s, p)
                    if from().drawtiles ~= nil then
                      p = from():drawtiles(p);
                    end
                    local w,h = mainmenu.width,const.picheight;
                    drawing.draw_mask(p, 0, 0, w, h);
                    return p;
                  end
    ,ondraw     = function(s, pic)
                    local w,h = mainmenu.width,const.picheight;
                    local start = mainmenu.itempad;
                    local pad = start;
                    local size = mainmenu.itemheight;
                    local step = size + mainmenu.itemspace;
                    if s._lasty ~= nil then
                      drawing.clean(pic, w, s._lasty, const.picwidth-w, size);
                    end
                    if s.sel ~= nil then
                      local o = obj[s.sel];
                      if o ~= nil and o[1] ~= "" then
                        local y = (s.sel-1)*step + pad;
                        drawing.copy_sel(pic, w, y, const.picwidth - inventory.defaults.menuwidth, size);
                        drawing.copy_text(pic, rcall(o[2], o), w + 10, y + ((size-drawing.text_height())/2));
                        s._lasty = y;
                      end
                    end
                    for i,e in ipairs(obj) do
                      local col = "white";
                      drawing.clean(pic, 0, start, w, size);
                      local enabled = e[4] == nil or rcall(e[4],e);
                      if s.sel == i and e[1] ~= "" and enabled then
                         drawing.copy_sel(pic, 0, start, w, size);
                      end
                      local c = "white";
                      if not enabled then
                        c = "gray";
                      end
                      drawing.copy_text(pic, rcall(e[1], e), 20, start+((size-drawing.text_height())/2), c, mainmenu.itemfont, mainmenu.itemfontsize);
                      start = start + step;
                    end
                  end
  }
end

function mainmenu.gameover()
  return engine.gameroom {
     img        = const.path.gfx.."gameover.jpg"
    ,hideinv    = true
    ,obj        ={ xact("checkpoint", code [[ gamefile(const.path.game..prefs["last_episode"], true); ]]) }
    ,dsc        = txtc("{checkpoint|Загрузить последнюю контрольную точку}")
  }
end