-- Codename Dream9 by Basil Voronkov
-- Module 'popup'
---------------------------------------------
-- This module implements and animated and 
-- optionally clickable popup window that can be
-- used for events and status information.
require "lib/core"
require "lib/drawing"
require "lib/tact"
require "lib/const"

popup = {
  defaults = {
    popupx = 10,
    popupy = 12,
    anistep = 6,
    fontsize = 12,
    padx = 4,
    pady = 2
  }
}

function popup.init()
  tact_resident:add(
    "popup",
    function()
      if _syspopup ~= nil then
        if not _syspopup.rec and _syspopup.shown then
          _syspopup = nil;
        else 
          _syspopup.shown=true;
        end
      end
    end
  );  
  _syspopup = nil;
end

function popup.show(dsc, act, rec, snd)
  _syspopup = { dsc = dsc, act = act, rec = rec }
  if snd ~= nil then
    play_sound(snd);
  end
end

function popup.hide()
  _syspopup = nil;
end

function popup.room(o)
  local ontimer = o.ontimer;
  local onclick = o.onclick;
  local ondraw = o.ondraw;
  local left = o.left;
  
  local function clean_popup(s)
    if s.popupw_ ~= nil and s.popuph_ ~= nil then
      drawing.clean(s.pic, popup.defaults.popupx, s.popupy_, s.popupw_, s.popuph_);
    end
  end
  
  local function draw_popup(s)
    if _syspopup ~= nil then
      clean_popup(s);
      local c = const.textcolor;
      if s.popupsel_ then
        c = const.selcolor;
      end
      if s.popupy_ ~= nil then
        s.popupw_,s.popuph_ = drawing.textframe(s.pic, _syspopup.dsc, popup.defaults.popupx, s.popupy_, 
          popup.defaults.padx, popup.defaults.pady, false, 
          const.font, popup.defaults.fontsize, c);
      end
    end
  end
  
  local function calc_height(s)
    local len = 0;
    for _ in s:split("^") do
      len = len + 1;
    end
    return drawing.text_height(const.font, popup.defaults.fontsize)*len + popup.defaults.pady*2 + (len-1)*2;
  end
  
  local function popup_hover(s, x, y)
    y = y - 20;
    if s.popupw_ == nil or s.popuph_ == nil or _syspopup == nil or _syspopup.act == nil then
      return false;
    end
    return y >= s.popupy_ and y <= s.popupy_ + s.popuph_
      and x >= popup.defaults.popupx and x <= popup.defaults.popupx + s.popupw_;
  end
  
  o.ondraw      = function(s, pic)
                    if ondraw ~= nil then
                      ondraw(s, pic);
                    end
                    if _syspopup ~= nil then
                      draw_popup(s, pic);
                    end
                  end
  o.ontimer     = function(s, x, y)
                    if ontimer ~= nil then
                      ontimer(s, x, y);
                    end
                    if _syspopup ~= nil  then
                      local hover = popup_hover(s, x, y);
                      if s.popupy_ == nil then
                        s.popupy_ = 0-calc_height(_syspopup.dsc);
                      end
                      if hover ~= s.popupsel_ then
                        s.popupsel_ = hover;
                        draw_popup(s);
                      elseif s.popupy_ < popup.defaults.popupy then
                        clean_popup(s);
                        s.popupy_ = s.popupy_ + popup.defaults.anistep;
                        draw_popup(s);
                      end
                    end
                  end
  o.onclick     = function(s, x, y)
                    if onclick ~= nil then
                      onclick(s, x, y);
                    end
                    local hover = popup_hover(s, x, y);
                    if hover ~= s.popupsel_ then
                      s.popupsel_ = hover;
                      draw_popup(s);
                    end
                  end
  o.left        = function(s)
                    if left ~= nil then
                      left(s);
                    end
                    if s.popupw_ ~= nil and s.popuph_ ~= nil then
                      clean_popup(s);
                      s.popupy_ = nil;
                    end
                  end
  return o;
end