-- Codename Dream9 by Basil Voronkov
-- Module 'rooms'
---------------------------------------------
-- This module implements various high level
-- rooms based on the 'gameroom' primitive defined
-- in 'engine' module.
require "theme"
require "lib/engine"
require "lib/inventory"
require "lib/popup"
require "lib/const"
require "lib/table"
require "lib/achievement"

rooms = {
  defaults = {
    itempicx = 20,
    itempicy = 20,
    itemnamefont = const.fontb,
    itemnamesize = 14,
    itemnameshift = 10,
    itemnamepadx = 8,
    itemnamepady = 4,
  }
}

function rooms.init()  
  _activeitem = nil;
end

function scene(o)
  local ret = engine.gameroom(o);
  ret.fps       = 20;
  ret.alwaysdsc = o.alwaysdsc;
  ret.left      = function(s)
                    s._seen = true;
                  end
  ret.showdsc   = function(s)
                    return s.alwaysdsc or not s._seen;
                  end
  ret.onclickr  = function(s)
                    _activeitem = nil;
                    theme.set("scr.gfx.cursor.normal", const.cursor);
                    walk(here());
                  end
  return achievement.room(activeitem.room(popup.room(navigation.room(ret))));
end

function cutscene(o)
  local loc = o.next;
  if loc ~= nil and is_string(loc) then
    loc = walk_(loc);
  end
  if loc ~= nil then
    o.obj = { xact("next", loc) };
  end
  if o.dsc ~= nil then
    o.dsc = o.dsc.."{next|"..img(const.path.ui.."next.png")..txtnb("    ").."}";
  end
  o.hideinv = true;
  return engine.gameroom(o);
end

function animation(o)
  return engine.gameroom {
     fps        = o.fps
    ,enter      = o.enter
    ,_frame     = 0
    ,frames     = o.frames
    ,len        = table.len(o.frames)
    ,img        = o.frames[1]
    ,next_      = o.next
    ,fading     = 0
    ,hideinv    = true
    ,ontimer    = function(s)
                    s._frame = s._frame + 1;
                    if s._frame > s.len then
                      walk(s.next_);
                    else
                      local fn = s.frames[s._frame];
                      local p = sprite.load(fn);
                      sprite.copy(p, s.pic, 0, 0);
                      sprite.free(p);
                    end
                  end
  }
end

function popup_if(r)
  local ur = rcall(r);
  if ur ~= nil and is_string(ur) then
    popup.show(ur);
    walk(here());
  end
end

function staticitem(o)
  local ret = engine.item {
     nam        = ""
    ,obj        = o.obj
    ,showonce   = o.showonce
    ,cnd_       = o.cnd
    ,raw_       = o.dsc
    ,dsc        = function(s)
                    if s.cnd_ == nil or rcall(s.cnd_,s) then
                      if s.showonce and s._seen then
                        return s.raw_:normalize();
                      else
                        return s.raw_;
                      end
                    end
                  end
    ,inf_       = o.inf
    ,img_       = o.img
    ,act_       = o.act
    ,room_      = o.room_
    ,act        = function(s)
                    if _activeitem ~= nil then
                      popup_if(ref(_activeitem):use(deref(s)));
                      _activeitem = nil;
                      theme.set("scr.gfx.cursor.normal", const.cursor);
                    else
                      s._seen = true;
                      if s.act_ ~= nil then
                        popup_if(rcall(s.act_, s));
                      end
                    end
                  end
  }
  return ret;
end

function takeitem(o)
  local activate = o.activate;
  return engine.item {
     nam        = o.nam
    ,ico        = o.ico
    ,img        = o.img
    ,dsc        = o.dsc
    ,inf        = o.inf
    ,tak_       = o.tak
    ,activate   = function(s)
                    if activate == nil then
                      _activeitem = deref(s);
                      theme.set("scr.gfx.cursor.normal", s.img);
                    else
                      activate(s);
                    end
                  end
    ,take       = function(s)
                    local res = rcall(s.tak_, s);
                    inventory.add(s);
                    popup.show(res);
                    remove(s);
                    remove(s, from());
                  end
    ,use        = o.use
    ,act_       = o.act
    ,act        = function(s)
                    if _activeitem ~= nil then
                      popup_if(ref(_activeitem):use(deref(s)));
                      _activeitem = nil;
                      theme.set("scr.gfx.cursor.normal", const.cursor);
                    else
                      if s.act_ ~= nil then
                        rcall(s.act_, s);
                      end
                      s:take();
                      if not here().gen_ then
                        walk(here());
                      else
                        walkback();
                      end
                    end
                  end
  }
end

function tremor(o)
  local step = o.step;
  if step == nil then
    step = 4;
  end
  return cutscene {
     fps        = o.fps
    ,_stage     = 0
    ,_anyy      = 0
    ,hideinv    = true
    ,pic        = const.path.gfx.."black.jpg"
    ,img_       = o.img
    ,step_      = step
    ,shift_     = o.shift
    ,next_      = o.next
    ,ontimer    = function(s, x, y)
                    if s.imgpic_ == nil then
                      s.imgpic_ = sprite.load(s.img_);
                    end
                    if s._stage == 0 and s._anyy > -s.shift_ then
                      s._anyy = s._anyy - s.step_;
                      if s._anyy == -s.shift_ then
                        s._stage = 1;
                      end
                    elseif s._stage == 1 and s._anyy < s.shift_ then
                      s._anyy = s._anyy + s.step_;
                      if s._anyy == s.shift_ then
                        s._stage = 2;
                      end
                    elseif s._stage == 2 and s._anyy > -s.step_ then
                      s._anyy = s._anyy - s.step_;
                      if s._anyy < 0 then
                        s._stage = 0;
                        s._anyy = 0;
                        sprite.free(s.imgpic_);
                        s.imgpic_ = nil;
                        if is_function(s.next_) then
                          s:next_();
                        else
                          walk(s.next_);
                        end
                      end
                    end
                    sprite.copy(s.imgpic_, s.pic, 0, s._anyy);
                    local boxh = math.abs(s._anyy);
                    local boxy = 0;
                    if s._anyy < 0 then
                      boxy = const.picheight + s._anyy;
                    end
                    local b = sprite.box(const.picwidth, boxh, "black");
                    sprite.copy(b, s.pic, 0, boxy);
                    sprite.free(b);
                  end
  }
end

function tremor2(o)
  local step = o.step;
  if step == nil then
    step = 4;
  end
  return engine.gameroom {
     fps        = o.fps
    ,hideinv    = true
    ,_stage     = 0
    ,_anyx      = 0
    ,pic        = const.path.gfx.."black.jpg"
    ,img_       = o.img
    ,step_      = step
    ,max_       = o.cycles
    ,cycles_    = 0
    ,next_      = o.next
    ,shift_     = o.shift
    ,ontimer    = function(s, x, y)
                    if s.imgpic_ == nil then
                      s.imgpic_ = sprite.load(s.img_);
                    end
                    if s._stage == 0 and s._anyx > -s.shift_ then
                      s._anyx = s._anyx - s.step_;
                      if s._anyx == -s.shift_ then
                        s._stage = 1;
                      end
                    elseif s._stage == 1 and s._anyx < s.shift_ then
                      s._anyx = s._anyx + s.step_;
                      if s._anyx == s.shift_ then
                        s._stage = 2;
                      end
                    elseif s._stage == 2 and s._anyx > -s.step_ then
                      s._anyx = s._anyx - s.step_;
                      if s._anyx < 0 then
                        s._stage = 0;
                        s._anyx = 0;
                        s.cycles_ = s.cycles_ + 1;
                        if s.cycles_ == s.max_ then
                          sprite.free(s.imgpic_);
                          s.imgpic_ = nil;
                          if is_function(s.next_) then
                            s:next_();
                          else
                            walk(s.next_);
                          end
                        end
                      end
                    end
                    local xx = s._anyx+20;
                    sprite.copy(s.imgpic_, xx, 0, const.picwidth, const.picheight, s.pic, 0, 0);
                  end
  }
end

function dizzy(o)
  local step = o.step;
  if step == nil then
    step = 4;
  end
  return cutscene {
     fps        = o.fps
    ,hideinv    = true
    ,_stage     = 0
    ,_anyx      = 0
    ,img        = o.bg
    ,img_       = o.img
    ,step_      = step
    ,shift_     = o.shift
    ,next_      = o.next
    ,ontimer    = function(s, x, y)
                    if s.imgpic_ == nil then
                      s.imgpic_ = sprite.load(s.img_);
                    end
                    if s._stage == 0 and s._anyx > -s.shift_ then
                      s._anyx = s._anyx - s.step_;
                      if s._anyx == -s.shift_ then
                        s._stage = 1;
                      end
                    elseif s._stage == 1 and s._anyx < s.shift_ then
                      s._anyx = s._anyx + s.step_;
                      if s._anyx == s.shift_ then
                        s._stage = 2;
                      end
                    elseif s._stage == 2 and s._anyx > -s.step_ then
                      s._anyx = s._anyx - s.step_;
                      if s._anyx < 0 then
                        s._stage = 0;
                        s._anyx = 0;
                        sprite.free(s.imgpic_);
                        s.imgpic_ = nil;
                        if is_function(s.next_) then
                          s:next_();
                        else
                          walk(s.next_);
                        end
                      end
                    end
                    sprite.compose(s.imgpic_, 0, 0, const.picwidth, const.picheight, s.pic, s._anyx, 0, 50);
                  end
  }
end

function pause(o)
  local pic = o.img;
  if pic == nil then
    pic = const.path.gfx.."black.jpg"
  end
  return engine.gameroom {
     img        = pic
    ,dsc        = o.dsc
    ,fading     = o.fading
    ,fps        = 1000/o.time
    ,next_      = o.next
    ,entered    = o.entered
    ,enter      = o.enter
    ,hideinv    = true
    ,ontimer    = function(s)
                    if is_function(s.next_) then
                      s:next_();
                    else
                      walk(s.next_);
                    end
                  end
  }
end

function itemroom(o)
  local activate = o.activate;
  local obj = table.append(o.obj, xact("back", walkback));
  local ret = engine.gameroom {
     gen_       = true
    ,hideinv    = true
    ,nomask     = true
    ,inf_       = o.inf
    ,img_       = o.img
    ,dsc        = function(s)
                    local d = rcall(s.inf_, o);
                    return d.."{back|"..img(const.path.ui.."next.png")..txtnb("    ").."}";
                  end
    ,img        = function(s)
                    return rcall(engine.previous_pic(s), s);
                  end
    ,nam_       = o.nam
    ,obj        = obj
    ,exit       = o.exit
    ,ondrawbg   = function(s, p)
                    drawing.draw_mask(p, 0, 0, const.picwidth, const.picheight, "black", 150);
                    local pic = drawing.load_pic(rcall(s.img_,s), false);
                    drawing.copy(pic, p, 20, 20);
                    sprite.free(pic);
                    return p;
                  end
    ,activate   = function(s)
                    if activate == nil then
                      activeitem.set(deref(s));
                    else
                      activate(s);
                    end
                  end
  }
  return ret;
end
