-- Codename Dream9 by Basil Voronkov
-- Module 'string'
---------------------------------------------
-- String extension functions including
-- string parsing functions.
require "lib/core"

function string:unilen()
  local _, count = string.gsub(self, "[^\128-\193]", "");
  return count;
end

function string:charat(i)
  local c = 0;
  for uchar in string.gfind(self, "([%z\1-\127\194-\244][\128-\191]*)") do
    c = c + 1;
    if c == i then
      return uchar;
    end
  end
end

function string:startsWith(start)
   return string.sub(self,1,string.unilen(start))==start
end

function string:split(pat)
  pat = pat or "%s+";
  local st, g = 1, self:gmatch("()("..pat..")")
  local function getter(segs, seps, sep, cap1, ...)
    st = sep and seps + #sep
    return self:sub(segs, (seps or 0) - 1), cap1 or sep, ...
  end
  return function() 
    if st then 
      return getter(st, g()) 
    end 
  end
end

function string:lines()
  local arr = self:split("^");
  local tab = {};
  local count = 0;  
  for e in arr do
    tab[count] = e;
    count = count + 1;
  end  
  return tab;
end

function string:parse(apply)
  return string.gsub(self, "\\?[\\{}]", { ["{"] = "\001", ["}"] = "\002" })
    :gsub("\001([^\002]+)\002", apply);
end

function string:expand()
  return self:parse(function(s) return tostring(apply(s)) end);
end

function string:normalize()
  local function last(s)
    local ret = "";
    for e in s:split("|") do
      if e == "next" then
        return "";
      end
      ret = e;
    end
    return ret;
  end
  local str = self:gsub("%s+", " ");
  return str:parse(function(s) return last(s) end);
end